// 发布预处理脚本

process.env.BABEL_ENV = 'production';
process.env.NODE_ENV = 'production';

const fs = require('fs');
const params = require('../package.json');
const webpack = require("webpack");
const path = require('path');

const files = fs.readdirSync('../build');
let jsName;
// let cssName;
files.forEach(filename => {
  if (/^main.*\.js$/.test(filename)) {
    jsName = filename;
  }
  // if (/^main.*\.css$/.test(filename)) {
  //   cssName = filename;
  // }
});

if (!jsName) {
  throw new Error('未找到编译打包后的css或js文件，请先在根目录运行 yarn build!')
}

const homepage = params.homepage;

const jsPath = homepage + jsName; // 全路径js

const fileString = getRootString(jsPath);

fs.writeFileSync('./src/components/Root.jsx', fileString);


function getRootString(jsPath) {
  return `// @flow strict

import * as React from 'react';
import load from 'load-script';

type Props = {
  children?: React.Node
};

type State = {
  hasLoad: boolean
};

class Root extends React.Component<Props, State> {

  state = {
    hasLoad: false
  };

  componentDidMount() {
    // 插入必要的第三方样式库
    load('${jsPath}', () => {
      this.setState({hasLoad: true})
    })
  }

  componentWillUnmount() {
    const portal = document.getElementById('react-v2-modal');
    if (portal && document.body) document.body.removeChild(portal);
  }

  render() {
    const {children} = this.props;
    const {hasLoad} = this.state;
    return (
      <div>
        {hasLoad && children}
      </div>
    );
  }
}

export default Root;`
}


const compiler = webpack({

  mode: 'production',

  entry: {
    main: './src/index.js'
  },

  output: {
    filename: 'bundle.js',
    path: path.resolve(__dirname, 'build'),
    library: require('./package.json').name,
    libraryTarget: 'umd',
    umdNamedDefine: true,
  },

  module: {
    rules: [
      {
        test: /\.(js|jsx)$/,
        include: path.resolve(__dirname, 'src'),
        use: 'babel-loader'
      }
    ]
  },

  externals: [
    {
      react: {
        root: 'React',
        commonjs2: 'react',
        commonjs: ['react'],
        amd: 'react',
      },
    },
    {
      'react-dom': {
        root: 'ReactDOM',
        commonjs2: 'react-dom',
        commonjs: ['react-dom'],
        amd: 'react-dom',
      },
    },
  ],

  resolve: {
    extensions: ['.web.js', '.mjs', '.js', '.json', '.web.jsx', '.jsx'],
  },

});

compiler.run((err, stats) => {
  console.log(err)
});
